import _toConsumableArray from 'babel-runtime/helpers/toConsumableArray';
import _typeof from 'babel-runtime/helpers/typeof';
import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React from 'react';
import PropTypes from 'prop-types';
import html from 'react-inner-html';
import stringifyObject from 'stringify-object';

var Terminal = function (_React$Component) {
  _inherits(Terminal, _React$Component);

  function Terminal(props) {
    _classCallCheck(this, Terminal);

    var _this = _possibleConstructorReturn(this, (Terminal.__proto__ || Object.getPrototypeOf(Terminal)).call(this, props));

    _this.state = {
      tracker: randstr(),
      commands: {},
      stdout: [],
      history: [],
      historyPosition: null
    };

    _this._getTerminalNode = _this._getTerminalNode.bind(_this);
    _this.focusTerminal = _this.focusTerminal.bind(_this);
    _this.validateCommands = _this.validateCommands.bind(_this);
    _this.showWelcomeMessage = _this.showWelcomeMessage.bind(_this);
    _this.showHelp = _this.showHelp.bind(_this);
    _this.pushToStdout = _this.pushToStdout.bind(_this);
    _this.getStdout = _this.getStdout.bind(_this);
    _this.clearStdout = _this.clearStdout.bind(_this);
    _this.processCommand = _this.processCommand.bind(_this);
    _this.handleInput = _this.handleInput.bind(_this);
    return _this;
  }

  _createClass(Terminal, [{
    key: '_getTerminalNode',
    value: function _getTerminalNode() {
      var _this2 = this;

      var elements = document.getElementsByName('react-console-emulator__input');

      // Foolproofing in case there are other elements with the same name
      if (elements.length > 1) return Array.from(elements).filter(function (el) {
        return el.dataset.input === _this2.state.tracker;
      })[0];else return elements[0];
    }

    /**
     * Manually push to the stdout of a terminal. Use with caution.
     * @param {String} message The message to output to the terminal. If not using safe mode, make sure to XSS-proof this.
     * @param {HTMLElement} contentElement The content element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {HTMLElement} inputElement The input element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {HTMLElement} inputAreaElement The input area element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {Boolean} dangerMode If true, set output content with innerHTML. Dangerous.
     */

  }, {
    key: 'focusTerminal',
    value: function focusTerminal() {
      this._getTerminalNode().focus();
    }
  }, {
    key: 'validateCommands',
    value: function validateCommands() {
      var immutables = ['help', 'clear'];
      var commands = this.props.commands;

      var defaultCommands = {
        help: {
          description: 'Show a list of available commands.',
          fn: this.showHelp
        },
        clear: {
          description: 'Empty the terminal window.',
          explicitExec: true,
          fn: this.clearStdout
        }
      };

      var validCommands = void 0;

      // Pre-register defaults
      if (!this.props.noDefaults) validCommands = _extends({}, defaultCommands);else validCommands = {};

      for (var c in commands) {
        // Check that command contains a function
        if (typeof commands[c].fn !== 'function') throw new Error('\'fn\' property of command \'' + c + '\' is invalid; expected \'function\', got \'' + _typeof(commands[c].fn) + '\'');

        // Check that the command does not attempt to override immutables
        if (!this.props.noDefaults && immutables.includes(c)) throw new Error('Attempting to overwrite default command \'' + immutables[immutables.indexOf(c)] + '\'; cannot override default commands');

        // Add description if missing
        if (!commands[c].description) commands[c].description = 'None';
        validCommands[c] = commands[c];
      }

      this.setState({ commands: validCommands });
    }
  }, {
    key: 'showWelcomeMessage',
    value: function showWelcomeMessage() {
      var _this3 = this;

      var msg = this.props.welcomeMessage;

      if (typeof msg === 'boolean') this.pushToStdout('Welcome to the React terminal! Type \'help\' to get a list of commands.');else if (Array.isArray(msg)) msg.map(function (item) {
        return _this3.pushToStdout(item);
      });else this.pushToStdout(msg);
    }
  }, {
    key: 'showHelp',
    value: function showHelp() {
      for (var c in this.state.commands) {
        var cmdObj = this.state.commands[c];
        var usage = cmdObj.usage ? ' - ' + cmdObj.usage : '';
        this.pushToStdout(c + ' - ' + cmdObj.description + usage);
      }
    }
  }, {
    key: 'pushToStdout',
    value: function pushToStdout(message, rawInput) {
      var stdout = this.state.stdout;
      var history = this.state.history;

      stdout.push(message);

      if (rawInput) {
        history.push(rawInput);
        this.setState({ stdout: stdout, history: history, historyPosition: null });
      } else this.setState({ stdout: stdout });
    }
  }, {
    key: 'getStdout',
    value: function getStdout() {
      var _this4 = this;

      var lineStyle = {
        margin: '0',
        lineHeight: '21px'
      };

      return this.state.stdout.map(function (line, i) {
        var safe = React.createElement(
          'p',
          { key: i, style: lineStyle },
          line
        );
        var dangerous = React.createElement('p', _extends({ key: i, style: lineStyle }, html(line)));

        return _this4.props.dangerMode ? dangerous : safe;
      });
    }
  }, {
    key: 'clearStdout',
    value: function clearStdout() {
      this.setState({ stdout: [] });
    }
  }, {
    key: 'clearInput',
    value: function clearInput() {
      this.setState({ historyPosition: null });
      this._getTerminalNode().value = '';
    }
  }, {
    key: 'processCommand',
    value: function processCommand() {
      var rawInput = this._getTerminalNode().value;

      var input = rawInput.split(' ');
      var command = input.splice(0, 1); // Removed portion is returned...
      var args = input; // ...and the rest can be used

      if (!this.props.noAutomaticStdout) {
        if (!this.props.noHistory) this.pushToStdout((this.props.promptLabel || '$') + ' ' + rawInput, rawInput);else this.pushToStdout((this.props.promptLabel || '$') + ' ' + rawInput);
      }

      if (rawInput) {
        var cmdObj = this.state.commands[command];

        if (!cmdObj) this.pushToStdout(this.props.errorText ? this.props.errorText.replace(/\[command\]/gi, command) : 'Command \'' + command + '\' not found!');else {
          this.pushToStdout(cmdObj.fn.apply(cmdObj, _toConsumableArray(args)));
          if (cmdObj.explicitExec) cmdObj.fn.apply(cmdObj, _toConsumableArray(args));
        }
      }

      this.clearInput();
    }
  }, {
    key: 'scrollHistory',
    value: function scrollHistory(direction) {
      var history = cleanArray(this.state.history).reverse(); // Clean empty items and reverse order to ease position tracking
      var position = this.state.historyPosition;
      var termNode = this._getTerminalNode();

      if (!this.state.noAutomaticStdout && history.length > 0) {
        // Only run if history is non-empty and in use
        switch (direction) {
          case 'up':
            if (position === null) {
              // If not touched, get most recent
              termNode.value = history[0];
              this.setState({ historyPosition: 0 });
            } else if (position + 1 === history.length) {
              // If the last item will be reached on this press, get first entry and decrement position by 1 to avoid confusing downscroll
              termNode.value = history[history.length - 1];
              this.setState({ historyPosition: history.length - 1 });
            } else {
              var atBottom = position - 1 === -1; // -1 for zero-based index

              // If at last item, increment by one more to avoid showing the same item twice
              termNode.value = atBottom ? history[position + 1] : history[position];
              this.setState({ historyPosition: atBottom ? position + 2 : position + 1 });
            }
            break;
          case 'down':
            if (position === null || !history[position]) {
              // If at initial or out of range, clear (Unix-like behaviour)
              termNode.value = '';
              this.setState({ historyPosition: null });
            } else if (position - 1 === -1) {
              // If position will go negative on this press, bottom has been reached - reset
              termNode.value = history[0];
              this.setState({ historyPosition: null });
            } else {
              var reachedFirst = position + 1 === history.length; // +1 for zero-based index

              // If at first item, decrement by one more to avoid showing the same item twice
              termNode.value = reachedFirst ? history[position - 1] : history[position];
              this.setState({ historyPosition: reachedFirst ? position - 2 : position - 1 });
            }
            break;
        }
      }
    }
  }, {
    key: 'handleInput',
    value: function handleInput(event) {
      switch (event.key) {
        case 'Enter':
          this.processCommand();break;
        case 'ArrowUp':
          this.scrollHistory('up');break;
        case 'ArrowDown':
          this.scrollHistory('down');break;
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      var oldCommands = stringifyObject(prevProps.commands);
      var currentCommands = stringifyObject(this.props.commands);

      // There was a change in commands
      if (oldCommands !== currentCommands) {
        // Re-validate
        this.validateCommands();
      }
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.validateCommands();
      if (this.props.welcomeMessage) this.showWelcomeMessage();
      if (this.props.autoFocus) this.focusTerminal();
    }
  }, {
    key: 'render',
    value: function render() {
      var styles = {
        container: {
          minWidth: '500x',
          minHeight: '300px',
          maxWidth: '100%', // Fill parent before overflowing
          maxHeight: '100%', // Fill parent before overflowing
          borderRadius: '5px',
          background: this.props.background || '#212121',
          backgroundSize: this.props.backgroundSize || 'cover',
          overflow: 'auto',
          cursor: 'text'
        },
        content: {
          padding: '20px',
          height: '100%',
          color: this.props.textColor || '#FFFFFF',
          fontFamily: '\'Inconsolata\', monospace',
          fontSize: '15px'
        },
        inputArea: {
          display: 'inline-flex',
          width: '100%'
        },
        prompt: {
          paddingTop: '3px',
          color: this.props.promptLabelColor || '#EE9C34'
        },
        input: {
          border: '0',
          padding: '0 0 0 7px',
          margin: '0',
          flexGrow: '100',
          width: '100%',
          height: '22px',
          background: 'transparent',
          fontFamily: '\'Inconsolata\', monospace',
          fontSize: '15px',
          color: this.props.promptTextColor || '#F0BF81'
        }
      };

      return React.createElement(
        'div',
        {
          className: this.props.className || '',
          name: 'react-console-emulator',
          style: styles.container,
          onClick: this.focusTerminal
        },
        React.createElement(
          'div',
          {
            className: this.props.contentClassName || '',
            name: 'react-console-emulator__content',
            style: styles.content
          },
          this.getStdout(),
          React.createElement(
            'div',
            {
              className: this.props.inputAreaClassName || '',
              name: 'react-console-emulator__inputArea',
              style: styles.inputArea
            },
            React.createElement(
              'span',
              {
                className: this.props.promptLabelClassName || '',
                style: styles.prompt
              },
              this.props.promptLabel || '$'
            ),
            React.createElement('input', {
              className: this.props.inputClassName || '',
              name: 'react-console-emulator__input',
              'data-input': this.state.tracker,
              style: styles.input,
              type: 'text',
              onKeyDown: this.handleInput
            })
          )
        )
      );
    }
  }], [{
    key: 'manualPushToStdout',
    value: function manualPushToStdout(message, dangerMode, contentElement, inputElement, inputAreaElement) {
      var content = contentElement || document.getElementsByName('react-console-emulator__content')[0];
      var input = inputElement || document.getElementsByName('react-console-emulator__input')[0];
      var inputArea = inputAreaElement || document.getElementsByName('react-console-emulator__inputArea')[0];

      var messageElement = document.createElement('p');

      if (dangerMode) messageElement.innerHTML = message;else messageElement.innerText = message;

      messageElement.style = 'margin: 0px; line-height: 21px;';

      content.appendChild(messageElement);
      content.appendChild(inputArea);
      input.value = '';
      input.focus();
    }
  }]);

  return Terminal;
}(React.Component);

export default Terminal;


Terminal.propTypes = {
  background: PropTypes.string,
  backgroundSize: PropTypes.string,
  textColor: PropTypes.string,
  promptLabelColor: PropTypes.string,
  promptTextColor: PropTypes.string,
  className: PropTypes.string,
  contentClassName: PropTypes.string,
  inputAreaClassName: PropTypes.string,
  promptLabelClassName: PropTypes.string,
  inputClassName: PropTypes.string,
  autoFocus: PropTypes.bool,
  dangerMode: PropTypes.bool,
  noDefaults: PropTypes.bool,
  noAutomaticStdout: PropTypes.bool,
  noHistory: PropTypes.bool,
  welcomeMessage: PropTypes.oneOfType([PropTypes.bool, PropTypes.array, PropTypes.string]),
  promptLabel: PropTypes.string,
  errorText: PropTypes.string,
  commands: PropTypes.object.isRequired
};

function randstr() {
  var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  var str = '';

  for (var i = 0; i < 15; i++) {
    str += chars.charAt(~~(Math.random() * chars.length));
  }

  return str;
}

/**
 * Workaround to clean an array from 'ghost items'.
 * @param {Array} dirtyArray
 */
function cleanArray(dirtyArray) {
  var newArray = Array.from(dirtyArray);
  return newArray.filter(function (i) {
    return i !== undefined;
  });
}